import { useCallback, useEffect, useMemo, useState } from 'react';
import { AxiosError } from 'axios';
import { staticMessages } from '~/driven/utils/constants/staticMessages';
import useGetPlaceList from '../../common/hooks/useGetPlaceList';
import { PlaceFormState, placeTypes } from '../view/protocol';
import { createPlaceArguments } from '../model/submiCreatePlace';
import { ICreatePlaceVm, initialPlaceFormState } from './protocols';

const onChangeInputStateHandlers = (setFormState: React.Dispatch<React.SetStateAction<PlaceFormState>>) => {
  const handleNameSetState = (_name: string, newValue: string) =>
    setFormState((prev) => ({ ...prev, placeName: newValue }));

  const handlePlaceTypeSetState = (newValue: `${placeTypes}`) =>
    setFormState((prev) => ({ ...prev, placeTypes: newValue }));

  const handleParentIdSetState = (newValue: string, label: string) =>
    setFormState((prev) => ({ ...prev, parentId: { value: newValue, label } }));

  return {
    handleNameSetState,
    handlePlaceTypeSetState,
    handleParentIdSetState,
  };
};

const resetParentIdOnChangePlaceType = (setFormState: React.Dispatch<React.SetStateAction<PlaceFormState>>) => {
  setFormState((prev) => ({ ...prev, parentId: { label: '', value: '' } }));
};

const useCreatePlaceVm = (dependencies: ICreatePlaceVm) => {
  const { getInputOptions, createPlaceModel } = dependencies;
  const { getParentIds, getPlaceType } = getInputOptions;
  const [formState, setFormState] = useState<PlaceFormState>(initialPlaceFormState);
  const [error, setError] = useState<{ message: string; type: 'error' | 'success' }>({ message: '', type: 'error' });

  const { handleNameSetState, handleParentIdSetState, handlePlaceTypeSetState } = useMemo(
    () => onChangeInputStateHandlers(setFormState),
    [],
  );

  const handleSubmitCreatePlaceForm = async (e: React.FormEvent) => {
    e.preventDefault();
    try {
      const { parentId, placeName, placeTypes: placeType } = formState;
      if (!placeName || !placeName) throw new Error(staticMessages.global.errors.inputPlace);
      const newPlaceData: createPlaceArguments = {
        parentId: parentId.value,
        placeName,
        placeType,
      };
      const response = await createPlaceModel(newPlaceData);
      if (!response) throw new Error(staticMessages.service.errors[500]);
      setError({
        message: staticMessages.global.success.createPlace,
        type: 'success',
      });
    } catch (errorExc) {
      if (errorExc instanceof AxiosError) {
        setError({ message: errorExc.response?.data?.description, type: 'error' });
      } else if (errorExc instanceof Error) {
        setError({ message: errorExc.message, type: 'error' });
      }
    }
  };
  useEffect(() => resetParentIdOnChangePlaceType(setFormState), [formState.placeTypes]);

  const getPlaceList = useCallback(() => useGetPlaceList()(), []);
  const places = getPlaceList();

  const placeTypesOptions = useMemo(() => getPlaceType(), []);
  const parentIdsOptions = useMemo(() => {
    if (!places.data) return [];
    return getParentIds(formState.placeTypes, places.data);
  }, [formState.placeTypes, places]);

  return {
    handleSubmitCreatePlaceForm,
    formStateData: {
      formState,
      error,
      setError,
      setFormState,
    },
    inputStateHandlers: {
      name: handleNameSetState,
      parentId: handleParentIdSetState,
      placeType: handlePlaceTypeSetState,
    },
    selectBoxOptions: {
      placeTypesOptions,
      parentIdsOptions,
      isParentIdLoading: places.isLoading,
    },
  };
};

export default useCreatePlaceVm;
